#include <SPI.h>
#include <MySensor.h>
#include <MyGateway.h>
#include "KakuRemoteReceiver.h"
#include "KakuRemoteTransmitter.h"
#include <Time.h>

#define PRESSDELAY 200

#define INCLUSION_MODE_TIME 1 // Number of minutes inclusion mode is enabled
#define INCLUSION_MODE_PIN 8 // Digital pin used for inclusion mode button
MyGateway gw(DEFAULT_CE_PIN, DEFAULT_CS_PIN, INCLUSION_MODE_TIME, INCLUSION_MODE_PIN,  6, 5, 4);

unsigned long lastCommand;
unsigned long lastTimestamp;
bool          commandPending = false;
bool          toggle         = false;
byte          lastSecond     = 0;

#define txPin 7
#define HOUSE_ADR 44444444

#define MAX_RECEIVE_LENGTH  100
char    inputString[MAX_RECEIVE_LENGTH] = "";    // A string to hold incoming commands from serial/ethernet interface
int     inputPos = 0;
boolean commandComplete = false;  // whether the string is complete

// Protocol handler
bool RFCodeAvailable = false;
unsigned long RFCode = 0;

void setup() {
  Serial.begin(115200);
  
  // Initialize receiver on interrupt 0 (= digital pin 2), calls the callback "showCode"
  // after 2 identical codes have been received in a row. (thus, keep the button pressed
  // for a moment)
  //
  // See the interrupt-parameter of attachInterrupt for possible values (and pins)
  // to connect the receiver.
  KakuRemoteReceiver::init(1, 1, showCode);
  
  lastTimestamp = millis();
  gw.begin();
}

void loop() {
    if (RFCodeAvailable)
    {
        processResult(RFCode);
        RFCodeAvailable = false;
    }
    gw.processRadioMessage();   
    if (commandComplete) {
      // A command wass issued from serial interface
      // We will now try to send it to the actuator
      if (inputString[0] == 'r')
      {  
          char *str, *value, *p;  
          str   = strtok_r(inputString, ";", &p);
          value = strtok_r(NULL, ";", &p);
          
          // This is an RF KaKu command
          RemoteTransmitter::sendCode(txPin, atoi(value), 240, 3);
      }
      else
      {
          gw.parseAndSend(inputString);
      }
      commandComplete = false;  
      inputPos = 0;
    }
}

void processResult(uint16_t res)
{
    // Last command more than 1 secons ago --> reset all 
    if ((lastTimestamp + 500) < millis()) {
        lastCommand    = res;
        lastTimestamp  = millis();
        commandPending = false;
        return;
    }
    // If we receive the same command twice whit in 1 second we have a go
    if (res == lastCommand) {
       if (!commandPending) {
          Serial.print("rf;");
          Serial.println(res);
       }
       commandPending = true;
    } else {
       commandPending = false;
    }
    lastCommand   = res;
    lastTimestamp = millis();
}

/*
  SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 */
void serialEvent() {
  while (Serial.available())  {
    // get the new byte:
    char inChar = (char)Serial.read(); 
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inputPos < MAX_RECEIVE_LENGTH-1 && !commandComplete) { 
      if (inChar == '\n' || inChar == '\r') {
        inputString[inputPos] = 0;
        commandComplete = true;
      } else {
        // add it to the inputString:
        inputString[inputPos] = inChar;
        inputPos++;
      }
    } else {
       // Incoming message too long. Throw away 
        inputPos = 0;
    }
  }
}


// Callback function is called only when a valid code is received.
void showCode(NewRemoteCode receivedCode) {
  // Note: interrupts are disabled. You can re-enable them if needed.
  if (receivedCode.oldStyle)
  {
    #ifdef MYDEBUG
    Serial.print("Code ");
    Serial.print(receivedCode.code);
    #endif
    RFCode = receivedCode.code;
    RFCodeAvailable = true;
  }
  else
  {
    #ifdef MYDEBUG
    // Print the received code.
    Serial.print("Addr ");
    Serial.print(receivedCode.address);
    
    if (receivedCode.groupBit) {
      Serial.print(" group");
    } else {
      Serial.print(" unit ");
      Serial.print(receivedCode.unit);
    }
    
    switch (receivedCode.switchType) {
      case NewRemoteCode::off:
        Serial.print(" off");
        break;
      case NewRemoteCode::on:
        Serial.print(" on");
        break;
      case NewRemoteCode::dim:
        Serial.print(" dim level ");
        Serial.print(receivedCode.dimLevel);
        break;
      case NewRemoteCode::on_with_dim:
        Serial.print(" on with dim level ");
        Serial.print(receivedCode.dimLevel);
        break;
    }
    #endif
    if (receivedCode.address == HOUSE_ADR)
    {
        RFCode = (receivedCode.groupBit ? 1000 : (receivedCode.unit + 1) * 10) + (receivedCode.switchType == NewRemoteCode::on ? 1 : 0);
        lastCommand = RFCode;
        RFCodeAvailable = true;
    }
  }
  #ifdef MUDEBUG
  Serial.print(", period: ");
  Serial.print(receivedCode.period);
  Serial.println("us.");
  #endif
}



